# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ErrorMessages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x45rrorMessages.proto\x12\x03xgt\"J\n\x05\x45rror\x12 \n\x04\x63ode\x18\x01 \x01(\x0e\x32\x12.xgt.ErrorCodeEnum\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0e\n\x06\x64\x65tail\x18\x03 \x01(\t*\xb6\x02\n\rErrorCodeEnum\x12\x0c\n\x08NO_ERROR\x10\x00\x12\x11\n\rGENERIC_ERROR\x10\x01\x12\x13\n\x0fNOT_IMPLEMENTED\x10\x02\x12\x12\n\x0eINTERNAL_ERROR\x10\x03\x12\x0c\n\x08IO_ERROR\x10\x04\x12\x17\n\x13SERVER_MEMORY_ERROR\x10\x05\x12\x14\n\x10\x43ONNECTION_ERROR\x10\x06\x12\x10\n\x0cSYNTAX_ERROR\x10\x07\x12\x0e\n\nTYPE_ERROR\x10\x08\x12\x0f\n\x0bVALUE_ERROR\x10\t\x12\x0e\n\nNAME_ERROR\x10\n\x12\x14\n\x10\x41RITHMETIC_ERROR\x10\x0b\x12\x1a\n\x16\x46RAME_DEPENDENCY_ERROR\x10\x0c\x12\x15\n\x11TRANSACTION_ERROR\x10\r\x12\x12\n\x0eSECURITY_ERROR\x10\x0e\x62\x06proto3')

_ERRORCODEENUM = DESCRIPTOR.enum_types_by_name['ErrorCodeEnum']
ErrorCodeEnum = enum_type_wrapper.EnumTypeWrapper(_ERRORCODEENUM)
NO_ERROR = 0
GENERIC_ERROR = 1
NOT_IMPLEMENTED = 2
INTERNAL_ERROR = 3
IO_ERROR = 4
SERVER_MEMORY_ERROR = 5
CONNECTION_ERROR = 6
SYNTAX_ERROR = 7
TYPE_ERROR = 8
VALUE_ERROR = 9
NAME_ERROR = 10
ARITHMETIC_ERROR = 11
FRAME_DEPENDENCY_ERROR = 12
TRANSACTION_ERROR = 13
SECURITY_ERROR = 14


_ERROR = DESCRIPTOR.message_types_by_name['Error']
Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), {
  'DESCRIPTOR' : _ERROR,
  '__module__' : 'ErrorMessages_pb2'
  # @@protoc_insertion_point(class_scope:xgt.Error)
  })
_sym_db.RegisterMessage(Error)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ERRORCODEENUM._serialized_start=105
  _ERRORCODEENUM._serialized_end=415
  _ERROR._serialized_start=28
  _ERROR._serialized_end=102
# @@protoc_insertion_point(module_scope)
