# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GraphTypesService.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import AdminService_pb2 as AdminService__pb2
from . import ErrorMessages_pb2 as ErrorMessages__pb2
from . import SchemaMessages_pb2 as SchemaMessages__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17GraphTypesService.proto\x12\x03xgt\x1a\x12\x41\x64minService.proto\x1a\x13\x45rrorMessages.proto\x1a\x14SchemaMessages.proto\"\x19\n\x08LabelSet\x12\r\n\x05label\x18\x01 \x03(\t\"\x90\x01\n\rFrameLabelMap\x12;\n\raccess_labels\x18\x01 \x03(\x0b\x32$.xgt.FrameLabelMap.AccessLabelsEntry\x1a\x42\n\x11\x41\x63\x63\x65ssLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.xgt.LabelSet:\x02\x38\x01\"q\n\x17GetVertexFramesResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12&\n\x0cvertex_frame\x18\x02 \x03(\x0b\x32\x10.xgt.VertexFrame\x12\x13\n\x0b\x65rror_frame\x18\x03 \x03(\t\"k\n\x15GetEdgeFramesResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12\"\n\nedge_frame\x18\x02 \x03(\x0b\x32\x0e.xgt.EdgeFrame\x12\x13\n\x0b\x65rror_frame\x18\x03 \x03(\t\"n\n\x16GetTableFramesResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12$\n\x0btable_frame\x18\x02 \x03(\x0b\x32\x0f.xgt.TableFrame\x12\x13\n\x0b\x65rror_frame\x18\x03 \x03(\t\"\xfe\x01\n\x16\x43reateNamespaceRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\x12+\n\x0f\x66rame_label_map\x18\x02 \x01(\x0b\x32\x12.xgt.FrameLabelMap\x12\x37\n\x06kwargs\x18\x03 \x03(\x0b\x32\'.xgt.CreateNamespaceRequest.KwargsEntry\x12)\n\x12row_label_universe\x18\x04 \x01(\x0b\x32\r.xgt.LabelSet\x1a?\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.xgt.ConfigValue:\x02\x38\x01\"4\n\x17\x43reateNamespaceResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\"\xba\x01\n\x16\x44\x65leteNamespaceRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\x12\x12\n\nforce_drop\x18\x02 \x01(\x08\x12\x37\n\x06kwargs\x18\x03 \x03(\x0b\x32\'.xgt.DeleteNamespaceRequest.KwargsEntry\x1a?\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.xgt.ConfigValue:\x02\x38\x01\"O\n\x17\x44\x65leteNamespaceResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12\x19\n\x11\x66ound_and_deleted\x18\x02 \x01(\x08\"\x16\n\x14GetNamespacesRequest\"J\n\x15GetNamespacesResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12\x16\n\x0enamespace_name\x18\x02 \x03(\t\"\x8b\x03\n\x12\x43reateFrameRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\x12\x1b\n\x06schema\x18\x02 \x01(\x0b\x32\x0b.xgt.Schema\x12\x15\n\rsource_vertex\x18\x03 \x01(\t\x12\x15\n\rtarget_vertex\x18\x04 \x01(\t\x12\x12\n\nsource_key\x18\x05 \x01(\t\x12\x12\n\ntarget_key\x18\x06 \x01(\t\x12 \n\x04type\x18\x07 \x01(\x0e\x32\x12.xgt.FrameTypeEnum\x12+\n\x0f\x66rame_label_map\x18\x08 \x01(\x0b\x32\x12.xgt.FrameLabelMap\x12\x33\n\x06kwargs\x18\t \x03(\x0b\x32#.xgt.CreateFrameRequest.KwargsEntry\x12)\n\x12row_label_universe\x18\n \x01(\x0b\x32\r.xgt.LabelSet\x1a?\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.xgt.ConfigValue:\x02\x38\x01\"@\n\x18GetFramesMetadataRequest\x12\x0c\n\x04name\x18\x01 \x03(\t\x12\x16\n\x0enamespace_name\x18\x02 \x01(\t\"X\n\x19GetFramesMetadataResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12 \n\tcontainer\x18\x02 \x03(\x0b\x32\r.xgt.AnyFrame\"\x9e\x01\n\x12\x44\x65leteFrameRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\x12\x33\n\x06kwargs\x18\x02 \x03(\x0b\x32#.xgt.DeleteFrameRequest.KwargsEntry\x1a?\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.xgt.ConfigValue:\x02\x38\x01\"K\n\x13\x44\x65leteFrameResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12\x19\n\x11\x66ound_and_deleted\x18\x02 \x01(\x08\")\n\x13GetFrameSizeRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\"?\n\x14GetFrameSizeResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12\x0c\n\x04size\x18\x02 \x01(\x04\"+\n\x15GetFrameLabelsRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\"`\n\x16GetFrameLabelsResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12+\n\x0f\x66rame_label_map\x18\x02 \x01(\x0b\x32\x12.xgt.FrameLabelMap\"0\n\x1aGetRowLabelUniverseRequest\x12\x12\n\nframe_name\x18\x01 \x01(\t\"[\n\x1bGetRowLabelUniverseResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12!\n\nrow_labels\x18\x02 \x01(\x0b\x32\r.xgt.LabelSet2\xf4\x06\n\x11GraphTypesService\x12R\n\x0fGetVertexFrames\x12\x1d.xgt.GetFramesMetadataRequest\x1a\x1e.xgt.GetFramesMetadataResponse\"\x00\x12P\n\rGetEdgeFrames\x12\x1d.xgt.GetFramesMetadataRequest\x1a\x1e.xgt.GetFramesMetadataResponse\"\x00\x12Q\n\x0eGetTableFrames\x12\x1d.xgt.GetFramesMetadataRequest\x1a\x1e.xgt.GetFramesMetadataResponse\"\x00\x12N\n\x0f\x43reateNamespace\x12\x1b.xgt.CreateNamespaceRequest\x1a\x1c.xgt.CreateNamespaceResponse\"\x00\x12N\n\x0f\x44\x65leteNamespace\x12\x1b.xgt.DeleteNamespaceRequest\x1a\x1c.xgt.DeleteNamespaceResponse\"\x00\x12H\n\rGetNamespaces\x12\x19.xgt.GetNamespacesRequest\x1a\x1a.xgt.GetNamespacesResponse\"\x00\x12H\n\x0b\x43reateFrame\x12\x17.xgt.CreateFrameRequest\x1a\x1e.xgt.GetFramesMetadataResponse\"\x00\x12\x42\n\x0b\x44\x65leteFrame\x12\x17.xgt.DeleteFrameRequest\x1a\x18.xgt.DeleteFrameResponse\"\x00\x12\x45\n\x0cGetFrameSize\x12\x18.xgt.GetFrameSizeRequest\x1a\x19.xgt.GetFrameSizeResponse\"\x00\x12K\n\x0eGetFrameLabels\x12\x1a.xgt.GetFrameLabelsRequest\x1a\x1b.xgt.GetFrameLabelsResponse\"\x00\x12Z\n\x13GetRowLabelUniverse\x12\x1f.xgt.GetRowLabelUniverseRequest\x1a .xgt.GetRowLabelUniverseResponse\"\x00\x62\x06proto3')



_LABELSET = DESCRIPTOR.message_types_by_name['LabelSet']
_FRAMELABELMAP = DESCRIPTOR.message_types_by_name['FrameLabelMap']
_FRAMELABELMAP_ACCESSLABELSENTRY = _FRAMELABELMAP.nested_types_by_name['AccessLabelsEntry']
_GETVERTEXFRAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetVertexFramesResponse']
_GETEDGEFRAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetEdgeFramesResponse']
_GETTABLEFRAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetTableFramesResponse']
_CREATENAMESPACEREQUEST = DESCRIPTOR.message_types_by_name['CreateNamespaceRequest']
_CREATENAMESPACEREQUEST_KWARGSENTRY = _CREATENAMESPACEREQUEST.nested_types_by_name['KwargsEntry']
_CREATENAMESPACERESPONSE = DESCRIPTOR.message_types_by_name['CreateNamespaceResponse']
_DELETENAMESPACEREQUEST = DESCRIPTOR.message_types_by_name['DeleteNamespaceRequest']
_DELETENAMESPACEREQUEST_KWARGSENTRY = _DELETENAMESPACEREQUEST.nested_types_by_name['KwargsEntry']
_DELETENAMESPACERESPONSE = DESCRIPTOR.message_types_by_name['DeleteNamespaceResponse']
_GETNAMESPACESREQUEST = DESCRIPTOR.message_types_by_name['GetNamespacesRequest']
_GETNAMESPACESRESPONSE = DESCRIPTOR.message_types_by_name['GetNamespacesResponse']
_CREATEFRAMEREQUEST = DESCRIPTOR.message_types_by_name['CreateFrameRequest']
_CREATEFRAMEREQUEST_KWARGSENTRY = _CREATEFRAMEREQUEST.nested_types_by_name['KwargsEntry']
_GETFRAMESMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetFramesMetadataRequest']
_GETFRAMESMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetFramesMetadataResponse']
_DELETEFRAMEREQUEST = DESCRIPTOR.message_types_by_name['DeleteFrameRequest']
_DELETEFRAMEREQUEST_KWARGSENTRY = _DELETEFRAMEREQUEST.nested_types_by_name['KwargsEntry']
_DELETEFRAMERESPONSE = DESCRIPTOR.message_types_by_name['DeleteFrameResponse']
_GETFRAMESIZEREQUEST = DESCRIPTOR.message_types_by_name['GetFrameSizeRequest']
_GETFRAMESIZERESPONSE = DESCRIPTOR.message_types_by_name['GetFrameSizeResponse']
_GETFRAMELABELSREQUEST = DESCRIPTOR.message_types_by_name['GetFrameLabelsRequest']
_GETFRAMELABELSRESPONSE = DESCRIPTOR.message_types_by_name['GetFrameLabelsResponse']
_GETROWLABELUNIVERSEREQUEST = DESCRIPTOR.message_types_by_name['GetRowLabelUniverseRequest']
_GETROWLABELUNIVERSERESPONSE = DESCRIPTOR.message_types_by_name['GetRowLabelUniverseResponse']
LabelSet = _reflection.GeneratedProtocolMessageType('LabelSet', (_message.Message,), {
  'DESCRIPTOR' : _LABELSET,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.LabelSet)
  })
_sym_db.RegisterMessage(LabelSet)

FrameLabelMap = _reflection.GeneratedProtocolMessageType('FrameLabelMap', (_message.Message,), {

  'AccessLabelsEntry' : _reflection.GeneratedProtocolMessageType('AccessLabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _FRAMELABELMAP_ACCESSLABELSENTRY,
    '__module__' : 'GraphTypesService_pb2'
    # @@protoc_insertion_point(class_scope:xgt.FrameLabelMap.AccessLabelsEntry)
    })
  ,
  'DESCRIPTOR' : _FRAMELABELMAP,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.FrameLabelMap)
  })
_sym_db.RegisterMessage(FrameLabelMap)
_sym_db.RegisterMessage(FrameLabelMap.AccessLabelsEntry)

GetVertexFramesResponse = _reflection.GeneratedProtocolMessageType('GetVertexFramesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVERTEXFRAMESRESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetVertexFramesResponse)
  })
_sym_db.RegisterMessage(GetVertexFramesResponse)

GetEdgeFramesResponse = _reflection.GeneratedProtocolMessageType('GetEdgeFramesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETEDGEFRAMESRESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetEdgeFramesResponse)
  })
_sym_db.RegisterMessage(GetEdgeFramesResponse)

GetTableFramesResponse = _reflection.GeneratedProtocolMessageType('GetTableFramesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTABLEFRAMESRESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetTableFramesResponse)
  })
_sym_db.RegisterMessage(GetTableFramesResponse)

CreateNamespaceRequest = _reflection.GeneratedProtocolMessageType('CreateNamespaceRequest', (_message.Message,), {

  'KwargsEntry' : _reflection.GeneratedProtocolMessageType('KwargsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CREATENAMESPACEREQUEST_KWARGSENTRY,
    '__module__' : 'GraphTypesService_pb2'
    # @@protoc_insertion_point(class_scope:xgt.CreateNamespaceRequest.KwargsEntry)
    })
  ,
  'DESCRIPTOR' : _CREATENAMESPACEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.CreateNamespaceRequest)
  })
_sym_db.RegisterMessage(CreateNamespaceRequest)
_sym_db.RegisterMessage(CreateNamespaceRequest.KwargsEntry)

CreateNamespaceResponse = _reflection.GeneratedProtocolMessageType('CreateNamespaceResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATENAMESPACERESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.CreateNamespaceResponse)
  })
_sym_db.RegisterMessage(CreateNamespaceResponse)

DeleteNamespaceRequest = _reflection.GeneratedProtocolMessageType('DeleteNamespaceRequest', (_message.Message,), {

  'KwargsEntry' : _reflection.GeneratedProtocolMessageType('KwargsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DELETENAMESPACEREQUEST_KWARGSENTRY,
    '__module__' : 'GraphTypesService_pb2'
    # @@protoc_insertion_point(class_scope:xgt.DeleteNamespaceRequest.KwargsEntry)
    })
  ,
  'DESCRIPTOR' : _DELETENAMESPACEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.DeleteNamespaceRequest)
  })
_sym_db.RegisterMessage(DeleteNamespaceRequest)
_sym_db.RegisterMessage(DeleteNamespaceRequest.KwargsEntry)

DeleteNamespaceResponse = _reflection.GeneratedProtocolMessageType('DeleteNamespaceResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETENAMESPACERESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.DeleteNamespaceResponse)
  })
_sym_db.RegisterMessage(DeleteNamespaceResponse)

GetNamespacesRequest = _reflection.GeneratedProtocolMessageType('GetNamespacesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETNAMESPACESREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetNamespacesRequest)
  })
_sym_db.RegisterMessage(GetNamespacesRequest)

GetNamespacesResponse = _reflection.GeneratedProtocolMessageType('GetNamespacesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETNAMESPACESRESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetNamespacesResponse)
  })
_sym_db.RegisterMessage(GetNamespacesResponse)

CreateFrameRequest = _reflection.GeneratedProtocolMessageType('CreateFrameRequest', (_message.Message,), {

  'KwargsEntry' : _reflection.GeneratedProtocolMessageType('KwargsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CREATEFRAMEREQUEST_KWARGSENTRY,
    '__module__' : 'GraphTypesService_pb2'
    # @@protoc_insertion_point(class_scope:xgt.CreateFrameRequest.KwargsEntry)
    })
  ,
  'DESCRIPTOR' : _CREATEFRAMEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.CreateFrameRequest)
  })
_sym_db.RegisterMessage(CreateFrameRequest)
_sym_db.RegisterMessage(CreateFrameRequest.KwargsEntry)

GetFramesMetadataRequest = _reflection.GeneratedProtocolMessageType('GetFramesMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMESMETADATAREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFramesMetadataRequest)
  })
_sym_db.RegisterMessage(GetFramesMetadataRequest)

GetFramesMetadataResponse = _reflection.GeneratedProtocolMessageType('GetFramesMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMESMETADATARESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFramesMetadataResponse)
  })
_sym_db.RegisterMessage(GetFramesMetadataResponse)

DeleteFrameRequest = _reflection.GeneratedProtocolMessageType('DeleteFrameRequest', (_message.Message,), {

  'KwargsEntry' : _reflection.GeneratedProtocolMessageType('KwargsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DELETEFRAMEREQUEST_KWARGSENTRY,
    '__module__' : 'GraphTypesService_pb2'
    # @@protoc_insertion_point(class_scope:xgt.DeleteFrameRequest.KwargsEntry)
    })
  ,
  'DESCRIPTOR' : _DELETEFRAMEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.DeleteFrameRequest)
  })
_sym_db.RegisterMessage(DeleteFrameRequest)
_sym_db.RegisterMessage(DeleteFrameRequest.KwargsEntry)

DeleteFrameResponse = _reflection.GeneratedProtocolMessageType('DeleteFrameResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFRAMERESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.DeleteFrameResponse)
  })
_sym_db.RegisterMessage(DeleteFrameResponse)

GetFrameSizeRequest = _reflection.GeneratedProtocolMessageType('GetFrameSizeRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMESIZEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFrameSizeRequest)
  })
_sym_db.RegisterMessage(GetFrameSizeRequest)

GetFrameSizeResponse = _reflection.GeneratedProtocolMessageType('GetFrameSizeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMESIZERESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFrameSizeResponse)
  })
_sym_db.RegisterMessage(GetFrameSizeResponse)

GetFrameLabelsRequest = _reflection.GeneratedProtocolMessageType('GetFrameLabelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMELABELSREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFrameLabelsRequest)
  })
_sym_db.RegisterMessage(GetFrameLabelsRequest)

GetFrameLabelsResponse = _reflection.GeneratedProtocolMessageType('GetFrameLabelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFRAMELABELSRESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetFrameLabelsResponse)
  })
_sym_db.RegisterMessage(GetFrameLabelsResponse)

GetRowLabelUniverseRequest = _reflection.GeneratedProtocolMessageType('GetRowLabelUniverseRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETROWLABELUNIVERSEREQUEST,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetRowLabelUniverseRequest)
  })
_sym_db.RegisterMessage(GetRowLabelUniverseRequest)

GetRowLabelUniverseResponse = _reflection.GeneratedProtocolMessageType('GetRowLabelUniverseResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETROWLABELUNIVERSERESPONSE,
  '__module__' : 'GraphTypesService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.GetRowLabelUniverseResponse)
  })
_sym_db.RegisterMessage(GetRowLabelUniverseResponse)

_GRAPHTYPESSERVICE = DESCRIPTOR.services_by_name['GraphTypesService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _FRAMELABELMAP_ACCESSLABELSENTRY._options = None
  _FRAMELABELMAP_ACCESSLABELSENTRY._serialized_options = b'8\001'
  _CREATENAMESPACEREQUEST_KWARGSENTRY._options = None
  _CREATENAMESPACEREQUEST_KWARGSENTRY._serialized_options = b'8\001'
  _DELETENAMESPACEREQUEST_KWARGSENTRY._options = None
  _DELETENAMESPACEREQUEST_KWARGSENTRY._serialized_options = b'8\001'
  _CREATEFRAMEREQUEST_KWARGSENTRY._options = None
  _CREATEFRAMEREQUEST_KWARGSENTRY._serialized_options = b'8\001'
  _DELETEFRAMEREQUEST_KWARGSENTRY._options = None
  _DELETEFRAMEREQUEST_KWARGSENTRY._serialized_options = b'8\001'
  _LABELSET._serialized_start=95
  _LABELSET._serialized_end=120
  _FRAMELABELMAP._serialized_start=123
  _FRAMELABELMAP._serialized_end=267
  _FRAMELABELMAP_ACCESSLABELSENTRY._serialized_start=201
  _FRAMELABELMAP_ACCESSLABELSENTRY._serialized_end=267
  _GETVERTEXFRAMESRESPONSE._serialized_start=269
  _GETVERTEXFRAMESRESPONSE._serialized_end=382
  _GETEDGEFRAMESRESPONSE._serialized_start=384
  _GETEDGEFRAMESRESPONSE._serialized_end=491
  _GETTABLEFRAMESRESPONSE._serialized_start=493
  _GETTABLEFRAMESRESPONSE._serialized_end=603
  _CREATENAMESPACEREQUEST._serialized_start=606
  _CREATENAMESPACEREQUEST._serialized_end=860
  _CREATENAMESPACEREQUEST_KWARGSENTRY._serialized_start=797
  _CREATENAMESPACEREQUEST_KWARGSENTRY._serialized_end=860
  _CREATENAMESPACERESPONSE._serialized_start=862
  _CREATENAMESPACERESPONSE._serialized_end=914
  _DELETENAMESPACEREQUEST._serialized_start=917
  _DELETENAMESPACEREQUEST._serialized_end=1103
  _DELETENAMESPACEREQUEST_KWARGSENTRY._serialized_start=797
  _DELETENAMESPACEREQUEST_KWARGSENTRY._serialized_end=860
  _DELETENAMESPACERESPONSE._serialized_start=1105
  _DELETENAMESPACERESPONSE._serialized_end=1184
  _GETNAMESPACESREQUEST._serialized_start=1186
  _GETNAMESPACESREQUEST._serialized_end=1208
  _GETNAMESPACESRESPONSE._serialized_start=1210
  _GETNAMESPACESRESPONSE._serialized_end=1284
  _CREATEFRAMEREQUEST._serialized_start=1287
  _CREATEFRAMEREQUEST._serialized_end=1682
  _CREATEFRAMEREQUEST_KWARGSENTRY._serialized_start=797
  _CREATEFRAMEREQUEST_KWARGSENTRY._serialized_end=860
  _GETFRAMESMETADATAREQUEST._serialized_start=1684
  _GETFRAMESMETADATAREQUEST._serialized_end=1748
  _GETFRAMESMETADATARESPONSE._serialized_start=1750
  _GETFRAMESMETADATARESPONSE._serialized_end=1838
  _DELETEFRAMEREQUEST._serialized_start=1841
  _DELETEFRAMEREQUEST._serialized_end=1999
  _DELETEFRAMEREQUEST_KWARGSENTRY._serialized_start=797
  _DELETEFRAMEREQUEST_KWARGSENTRY._serialized_end=860
  _DELETEFRAMERESPONSE._serialized_start=2001
  _DELETEFRAMERESPONSE._serialized_end=2076
  _GETFRAMESIZEREQUEST._serialized_start=2078
  _GETFRAMESIZEREQUEST._serialized_end=2119
  _GETFRAMESIZERESPONSE._serialized_start=2121
  _GETFRAMESIZERESPONSE._serialized_end=2184
  _GETFRAMELABELSREQUEST._serialized_start=2186
  _GETFRAMELABELSREQUEST._serialized_end=2229
  _GETFRAMELABELSRESPONSE._serialized_start=2231
  _GETFRAMELABELSRESPONSE._serialized_end=2327
  _GETROWLABELUNIVERSEREQUEST._serialized_start=2329
  _GETROWLABELUNIVERSEREQUEST._serialized_end=2377
  _GETROWLABELUNIVERSERESPONSE._serialized_start=2379
  _GETROWLABELUNIVERSERESPONSE._serialized_end=2470
  _GRAPHTYPESSERVICE._serialized_start=2473
  _GRAPHTYPESSERVICE._serialized_end=3357
# @@protoc_insertion_point(module_scope)
