# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import GraphTypesService_pb2 as GraphTypesService__pb2


class GraphTypesServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetVertexFrames = channel.unary_unary(
                '/xgt.GraphTypesService/GetVertexFrames',
                request_serializer=GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
                )
        self.GetEdgeFrames = channel.unary_unary(
                '/xgt.GraphTypesService/GetEdgeFrames',
                request_serializer=GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
                )
        self.GetTableFrames = channel.unary_unary(
                '/xgt.GraphTypesService/GetTableFrames',
                request_serializer=GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
                )
        self.CreateNamespace = channel.unary_unary(
                '/xgt.GraphTypesService/CreateNamespace',
                request_serializer=GraphTypesService__pb2.CreateNamespaceRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.CreateNamespaceResponse.FromString,
                )
        self.DeleteNamespace = channel.unary_unary(
                '/xgt.GraphTypesService/DeleteNamespace',
                request_serializer=GraphTypesService__pb2.DeleteNamespaceRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.DeleteNamespaceResponse.FromString,
                )
        self.GetNamespaces = channel.unary_unary(
                '/xgt.GraphTypesService/GetNamespaces',
                request_serializer=GraphTypesService__pb2.GetNamespacesRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetNamespacesResponse.FromString,
                )
        self.CreateFrame = channel.unary_unary(
                '/xgt.GraphTypesService/CreateFrame',
                request_serializer=GraphTypesService__pb2.CreateFrameRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
                )
        self.DeleteFrame = channel.unary_unary(
                '/xgt.GraphTypesService/DeleteFrame',
                request_serializer=GraphTypesService__pb2.DeleteFrameRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.DeleteFrameResponse.FromString,
                )
        self.GetFrameSize = channel.unary_unary(
                '/xgt.GraphTypesService/GetFrameSize',
                request_serializer=GraphTypesService__pb2.GetFrameSizeRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFrameSizeResponse.FromString,
                )
        self.GetFrameLabels = channel.unary_unary(
                '/xgt.GraphTypesService/GetFrameLabels',
                request_serializer=GraphTypesService__pb2.GetFrameLabelsRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetFrameLabelsResponse.FromString,
                )
        self.GetRowLabelUniverse = channel.unary_unary(
                '/xgt.GraphTypesService/GetRowLabelUniverse',
                request_serializer=GraphTypesService__pb2.GetRowLabelUniverseRequest.SerializeToString,
                response_deserializer=GraphTypesService__pb2.GetRowLabelUniverseResponse.FromString,
                )


class GraphTypesServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetVertexFrames(self, request, context):
        """Vertex frames.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEdgeFrames(self, request, context):
        """Edge frames.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTableFrames(self, request, context):
        """Table frames.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNamespace(self, request, context):
        """Namespaces functions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteNamespace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNamespaces(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateFrame(self, request, context):
        """Multi-type functions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFrame(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFrameSize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFrameLabels(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRowLabelUniverse(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GraphTypesServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetVertexFrames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVertexFrames,
                    request_deserializer=GraphTypesService__pb2.GetFramesMetadataRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFramesMetadataResponse.SerializeToString,
            ),
            'GetEdgeFrames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEdgeFrames,
                    request_deserializer=GraphTypesService__pb2.GetFramesMetadataRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFramesMetadataResponse.SerializeToString,
            ),
            'GetTableFrames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTableFrames,
                    request_deserializer=GraphTypesService__pb2.GetFramesMetadataRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFramesMetadataResponse.SerializeToString,
            ),
            'CreateNamespace': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNamespace,
                    request_deserializer=GraphTypesService__pb2.CreateNamespaceRequest.FromString,
                    response_serializer=GraphTypesService__pb2.CreateNamespaceResponse.SerializeToString,
            ),
            'DeleteNamespace': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteNamespace,
                    request_deserializer=GraphTypesService__pb2.DeleteNamespaceRequest.FromString,
                    response_serializer=GraphTypesService__pb2.DeleteNamespaceResponse.SerializeToString,
            ),
            'GetNamespaces': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNamespaces,
                    request_deserializer=GraphTypesService__pb2.GetNamespacesRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetNamespacesResponse.SerializeToString,
            ),
            'CreateFrame': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateFrame,
                    request_deserializer=GraphTypesService__pb2.CreateFrameRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFramesMetadataResponse.SerializeToString,
            ),
            'DeleteFrame': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFrame,
                    request_deserializer=GraphTypesService__pb2.DeleteFrameRequest.FromString,
                    response_serializer=GraphTypesService__pb2.DeleteFrameResponse.SerializeToString,
            ),
            'GetFrameSize': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFrameSize,
                    request_deserializer=GraphTypesService__pb2.GetFrameSizeRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFrameSizeResponse.SerializeToString,
            ),
            'GetFrameLabels': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFrameLabels,
                    request_deserializer=GraphTypesService__pb2.GetFrameLabelsRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetFrameLabelsResponse.SerializeToString,
            ),
            'GetRowLabelUniverse': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRowLabelUniverse,
                    request_deserializer=GraphTypesService__pb2.GetRowLabelUniverseRequest.FromString,
                    response_serializer=GraphTypesService__pb2.GetRowLabelUniverseResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'xgt.GraphTypesService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class GraphTypesService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetVertexFrames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetVertexFrames',
            GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
            GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEdgeFrames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetEdgeFrames',
            GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
            GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTableFrames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetTableFrames',
            GraphTypesService__pb2.GetFramesMetadataRequest.SerializeToString,
            GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateNamespace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/CreateNamespace',
            GraphTypesService__pb2.CreateNamespaceRequest.SerializeToString,
            GraphTypesService__pb2.CreateNamespaceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteNamespace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/DeleteNamespace',
            GraphTypesService__pb2.DeleteNamespaceRequest.SerializeToString,
            GraphTypesService__pb2.DeleteNamespaceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNamespaces(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetNamespaces',
            GraphTypesService__pb2.GetNamespacesRequest.SerializeToString,
            GraphTypesService__pb2.GetNamespacesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateFrame(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/CreateFrame',
            GraphTypesService__pb2.CreateFrameRequest.SerializeToString,
            GraphTypesService__pb2.GetFramesMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFrame(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/DeleteFrame',
            GraphTypesService__pb2.DeleteFrameRequest.SerializeToString,
            GraphTypesService__pb2.DeleteFrameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFrameSize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetFrameSize',
            GraphTypesService__pb2.GetFrameSizeRequest.SerializeToString,
            GraphTypesService__pb2.GetFrameSizeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFrameLabels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetFrameLabels',
            GraphTypesService__pb2.GetFrameLabelsRequest.SerializeToString,
            GraphTypesService__pb2.GetFrameLabelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRowLabelUniverse(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/xgt.GraphTypesService/GetRowLabelUniverse',
            GraphTypesService__pb2.GetRowLabelUniverseRequest.SerializeToString,
            GraphTypesService__pb2.GetRowLabelUniverseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
