# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: MetricsService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ErrorMessages_pb2 as ErrorMessages__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14MetricsService.proto\x12\x03xgt\x1a\x13\x45rrorMessages.proto\"\x16\n\x14MetricsStatusRequest\"Z\n\x15MetricsStatusResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error\x12&\n\x06status\x18\x02 \x01(\x0e\x32\x16.xgt.MetricsStatusEnum\"\x15\n\x13\x43learMetricsRequest\"1\n\x14\x43learMetricsResponse\x12\x19\n\x05\x65rror\x18\x01 \x03(\x0b\x32\n.xgt.Error*l\n\x11MetricsStatusEnum\x12\x1a\n\x16UNKNOWN_METRICS_STATUS\x10\x00\x12\x15\n\x11METRICS_COMPLETED\x10\x01\x12\x13\n\x0fMETRICS_RUNNING\x10\x02\x12\x0f\n\x0bMETRICS_OFF\x10\x03\x32\xa4\x01\n\x0eMetricsService\x12K\n\x10GetMetricsStatus\x12\x19.xgt.MetricsStatusRequest\x1a\x1a.xgt.MetricsStatusResponse\"\x00\x12\x45\n\x0c\x43learMetrics\x12\x18.xgt.ClearMetricsRequest\x1a\x19.xgt.ClearMetricsResponse\"\x00\x62\x06proto3')

_METRICSSTATUSENUM = DESCRIPTOR.enum_types_by_name['MetricsStatusEnum']
MetricsStatusEnum = enum_type_wrapper.EnumTypeWrapper(_METRICSSTATUSENUM)
UNKNOWN_METRICS_STATUS = 0
METRICS_COMPLETED = 1
METRICS_RUNNING = 2
METRICS_OFF = 3


_METRICSSTATUSREQUEST = DESCRIPTOR.message_types_by_name['MetricsStatusRequest']
_METRICSSTATUSRESPONSE = DESCRIPTOR.message_types_by_name['MetricsStatusResponse']
_CLEARMETRICSREQUEST = DESCRIPTOR.message_types_by_name['ClearMetricsRequest']
_CLEARMETRICSRESPONSE = DESCRIPTOR.message_types_by_name['ClearMetricsResponse']
MetricsStatusRequest = _reflection.GeneratedProtocolMessageType('MetricsStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICSSTATUSREQUEST,
  '__module__' : 'MetricsService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.MetricsStatusRequest)
  })
_sym_db.RegisterMessage(MetricsStatusRequest)

MetricsStatusResponse = _reflection.GeneratedProtocolMessageType('MetricsStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _METRICSSTATUSRESPONSE,
  '__module__' : 'MetricsService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.MetricsStatusResponse)
  })
_sym_db.RegisterMessage(MetricsStatusResponse)

ClearMetricsRequest = _reflection.GeneratedProtocolMessageType('ClearMetricsRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLEARMETRICSREQUEST,
  '__module__' : 'MetricsService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.ClearMetricsRequest)
  })
_sym_db.RegisterMessage(ClearMetricsRequest)

ClearMetricsResponse = _reflection.GeneratedProtocolMessageType('ClearMetricsResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLEARMETRICSRESPONSE,
  '__module__' : 'MetricsService_pb2'
  # @@protoc_insertion_point(class_scope:xgt.ClearMetricsResponse)
  })
_sym_db.RegisterMessage(ClearMetricsResponse)

_METRICSSERVICE = DESCRIPTOR.services_by_name['MetricsService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _METRICSSTATUSENUM._serialized_start=240
  _METRICSSTATUSENUM._serialized_end=348
  _METRICSSTATUSREQUEST._serialized_start=50
  _METRICSSTATUSREQUEST._serialized_end=72
  _METRICSSTATUSRESPONSE._serialized_start=74
  _METRICSSTATUSRESPONSE._serialized_end=164
  _CLEARMETRICSREQUEST._serialized_start=166
  _CLEARMETRICSREQUEST._serialized_end=187
  _CLEARMETRICSRESPONSE._serialized_start=189
  _CLEARMETRICSRESPONSE._serialized_end=238
  _METRICSSERVICE._serialized_start=351
  _METRICSSERVICE._serialized_end=515
# @@protoc_insertion_point(module_scope)
